"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mockGraphQLServer = mockGraphQLServer;
const tslib_1 = require("tslib");
const graphql_helix_1 = require("graphql-helix");
const nock_1 = tslib_1.__importDefault(require("nock"));
function mockGraphQLServer({ schema, host, path, intercept, method = 'POST', }) {
    const handler = async function (uri, body) {
        if (intercept) {
            intercept(this);
        }
        const uriObj = new URL(host + uri);
        const queryObj = {};
        for (const [key, val] of uriObj.searchParams.entries()) {
            queryObj[key] = val;
        }
        // Create a generic Request object that can be consumed by Graphql Helix's API
        const request = {
            body,
            headers: this.req.headers,
            method,
            query: queryObj,
        };
        // Extract the GraphQL parameters from the request
        const { operationName, query, variables } = (0, graphql_helix_1.getGraphQLParameters)(request);
        // Validate and execute the query
        const result = await (0, graphql_helix_1.processRequest)({
            operationName,
            query,
            variables,
            request,
            schema,
        });
        // processRequest returns one of three types of results depending on how the server should respond
        // 1) RESPONSE: a regular JSON payload
        // 2) MULTIPART RESPONSE: a multipart response (when @stream or @defer directives are used)
        // 3) PUSH: a stream of events to push back down the client for a subscription
        if (result.type === 'RESPONSE') {
            const headers = {};
            // We set the provided status and headers and just the send the payload back to the client
            for (const { name, value } of result.headers) {
                headers[name] = value;
            }
            return [result.status, result.payload, headers];
        }
        return [500, 'Not implemented'];
    };
    switch (method) {
        case 'GET':
            return (0, nock_1.default)(host).get(path).reply(handler);
        case 'POST':
            return (0, nock_1.default)(host).post(path).reply(handler);
    }
    return null;
}
